import discord
from cpl_core.configuration import ConfigurationABC
from cpl_core.logging import LoggerABC
from cpl_discord.events import OnMemberUpdateABC

from bot_core.configuration.feature_flags_enum import FeatureFlagsEnum
from bot_core.configuration.feature_flags_settings import FeatureFlagsSettings
from bot_data.model.server_config import ServerConfig
from modules.permission.abc.permission_service_abc import PermissionServiceABC


class PermissionOnMemberUpdateEvent(OnMemberUpdateABC):
    def __init__(self, config: ConfigurationABC, logger: LoggerABC, permission_service: PermissionServiceABC):
        OnMemberUpdateABC.__init__(self)
        self._config = config
        self._logger = logger
        self._permission_service = permission_service

    async def on_member_update(self, before: discord.Member, after: discord.Member):
        if before.guild is not None:
            server_config: ServerConfig = self._config.get_configuration(f"ServerConfig_{before.guild.id}")
            if not FeatureFlagsSettings.get_flag_from_dict(
                server_config.feature_flags, FeatureFlagsEnum.permission_module
            ):
                return

        self._logger.debug(__name__, f"Module {type(self)} started")

        if before.roles != after.roles:
            self._permission_service.on_member_update(before, after)
