import discord
from cpl_core.configuration import ConfigurationABC
from cpl_core.logging import LoggerABC
from cpl_discord.service import DiscordBotServiceABC
from cpl_query.extension import List

from bot_core.configuration.feature_flags_enum import FeatureFlagsEnum
from bot_core.configuration.feature_flags_settings import FeatureFlagsSettings
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.short_role_name_repository_abc import ShortRoleNameRepositoryABC
from bot_data.model.server_config import ServerConfig
from bot_data.model.short_role_name import ShortRoleName
from bot_data.model.short_role_name_position_enum import ShortRoleNamePositionEnum


class ShortRoleNameService:
    def __init__(
        self,
        config: ConfigurationABC,
        logger: LoggerABC,
        bot: DiscordBotServiceABC,
        server: ServerRepositoryABC,
        short_role_names: ShortRoleNameRepositoryABC,
    ):
        self._config = config
        self._logger = logger
        self._bot = bot
        self._server = server
        self._short_role_names = short_role_names

    async def check_short_role_names(self, member: discord.Member):
        self._logger.debug(__name__, f"Started short role name check for {member.id}")
        settings: ServerConfig = self._config.get_configuration(f"ServerConfig_{member.guild.id}")
        if not FeatureFlagsSettings.get_flag_from_dict(settings.feature_flags, FeatureFlagsEnum.short_role_name):
            self._logger.debug(__name__, f"Feature not active")
            return

        if member == member.guild.owner:
            return

        before = ""
        after = ""
        new_nick = member.nick if member.nick is not None else member.name
        role_ids = List(discord.Role, member.roles).select(lambda x: x.id)

        server = self._server.get_server_by_discord_id(member.guild.id)
        for short_role_name in self._short_role_names.get_short_role_names_by_server_id(server.id):
            short_role_name: ShortRoleName = short_role_name

            new_nick = new_nick.replace(f" [{short_role_name.short_name}]", "")
            new_nick = new_nick.replace(f"[{short_role_name.short_name}] ", "")
            new_nick = new_nick.replace(f"[{short_role_name.short_name}]", "")

            if short_role_name.role_id not in role_ids:
                continue

            if short_role_name.position == ShortRoleNamePositionEnum.before:
                before += f"[{short_role_name.short_name}] "
            elif short_role_name.position == ShortRoleNamePositionEnum.after:
                after += f" [{short_role_name.short_name}]"

        if before != "":
            new_nick = before + new_nick

        if after != "":
            new_nick = new_nick + after

        if member.nick == new_nick:
            return

        try:
            self._logger.debug(__name__, f"Update member {member.id}")
            await member.edit(nick=new_nick)
            self._logger.debug(__name__, f"Updated member {member.id} {member.name}")
        except Exception as e:
            self._logger.error(__name__, f"Renaming member {member.name} failed", e)
