import json
from datetime import datetime

from cpl_query.extension import List


class UserJoinedVoiceChannel:
    def __init__(self):
        self.JoinId = None
        self.UserId = None
        self.DiscordChannelId = None
        self.JoinedOn = None
        self.LeavedOn = None
        self.CreatedAt = None
        self.LastModifiedAt = None

    @property
    def ontime(self) -> int:
        return round((self.LeavedOn - self.JoinedOn).total_seconds() / 3600, 2)

    def from_dict(self, data: dict):
        self.JoinId = data["JoinId"]
        self.UserId = data["UserId"]
        self.DiscordChannelId = data["DiscordChannelId"]
        self.JoinedOn = datetime.strptime(data["JoinedOn"], "%Y-%m-%d %H:%M:%S.%f")
        self.LeavedOn = datetime.strptime(data["LeavedOn"], "%Y-%m-%d %H:%M:%S.%f")
        self.CreatedAt = data["CreatedAt"]
        self.LastModifiedAt = data["LastModifiedAt"]

    def __repr__(self):
        return f"{self.JoinId}: {self.UserId} {self.JoinedOn} {self.LeavedOn}"


def main():
    data = json.loads(open("UserJoinedVoiceChannel.json", "r").read())
    ujvcs = List(UserJoinedVoiceChannel)
    for x in data:
        ujvc = UserJoinedVoiceChannel()
        ujvc.from_dict(x)
        ujvcs.append(ujvc)

    users = ujvcs.order_by(lambda x: x.UserId).select(lambda x: x.UserId).distinct()
    groups = {}
    for user in users:
        groups[user] = List(UserJoinedVoiceChannel)
        groups[user].extend(ujvcs.where(lambda x: x.UserId == user))

    for group in groups:
        values: List[UserJoinedVoiceChannel] = groups[group]

        def find_overlaps(x: UserJoinedVoiceChannel) -> bool:
            for y in values:
                if x == y:
                    continue

                from collections import namedtuple

                Range = namedtuple("Range", ["start", "end"])
                r1 = Range(start=x.JoinedOn, end=x.LeavedOn)
                r2 = Range(start=y.LeavedOn, end=y.LeavedOn)
                latest_start = max(r1.start, r2.start)
                earliest_end = min(r1.end, r2.end)
                delta = (earliest_end - latest_start).days + 1
                overlap = max(0, delta)
                if overlap > 0:
                    return True

        overlaps = values.where(find_overlaps)
        if overlaps.count() == 0:
            continue

        print(overlaps.select(lambda x: x.JoinId))
        print(f"{group} has {overlaps.count()} of {values.count()}")
        ontime = overlaps.sum(lambda x: x.ontime)
        print(f"ontime: {ontime}")
        print(f"xp: {ontime * 8}")


if __name__ == "__main__":
    main()
