import { Injectable } from "@angular/core";
import { ActivatedRoute, ActivatedRouteSnapshot, CanActivate, Router, RouterStateSnapshot } from "@angular/router";
import { catchError } from "rxjs/operators";
import { AuthService } from "src/app/services/auth/auth.service";
import { ThemeService } from "src/app/services/theme/theme.service";
import { SidebarService } from "../../../../services/sidebar/sidebar.service";
import { MemberRoles } from "../../../../models/auth/auth-user.dto";
import { DataService } from "../../../../services/data/data.service";

@Injectable({
  providedIn: "root"
})
export class AuthGuard implements CanActivate {
  constructor(
    private router: Router,
    private authService: AuthService,
    private themeService: ThemeService,
    private sidebarService: SidebarService,
    private data: DataService,
    private route: ActivatedRoute
  ) {
  }

  async canActivate(route: ActivatedRouteSnapshot, state: RouterStateSnapshot): Promise<boolean> {
    if (!this.authService.getToken().token) {
      this.authService.logout();
      return false;
    }

    if (!await this.authService.isUserLoggedInAsync()) {
      this.authService.logout();
      return false;
    }

    const role = route.data["role"];
    const memberRole = route.data["memberRole"];
    if (role !== undefined) {
      this.authService.hasUserPermission(role).then(async hasPermission => {
        let authUser = await this.authService.getLoggedInUser();
        let isTechnician = authUser?.users?.map(u => u.isTechnician).filter(u => u) ?? [];

        if (!hasPermission && !isTechnician) {
          this.router.navigate(["/dashboard"]);
          return false;
        }
        return true;
      });
    }

    if (memberRole !== undefined) {
      let userHasAccess = false;
      let authUser = await this.authService.getLoggedInUser();
      let server = route.params["serverId"];

      if (!authUser || !authUser.users) {
        return true;
      }
      authUser?.users?.forEach(u => {
        if (u.server === +(server ?? 0)) {
          if (
            memberRole === MemberRoles.Moderator && u.isModerator ||
            memberRole === MemberRoles.Admin && u.isAdmin ||
            memberRole === MemberRoles.Technician && u.isTechnician
          ) {
            userHasAccess = true;
          }
        }
      });
      if (!userHasAccess) {
        this.router.navigate(["/dashboard"]);
        return false;
      }
      return true;
    }
    return true;
  }
}
